/*
 Copyright (c) 2007, Mike Kinney
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the 
 following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the 
 following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
 following disclaimer in the documentation and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

//
//  MKFacebook.h
//  MKAbeFook
//
//  Created by Mike on 10/11/06.
//  Copyright 2006 Mike Kinney. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#define MKXMLFacebookDataFormat 0
#define MKJSONFacebookDataFormat 1

@class MKLoginWindow;

extern NSString* const MKErrorConnection;   // if the connection has a problem
extern NSString* const MKErrorCancelled;    // if the user cancelled the operation
extern NSString* const MKErrorMissingParam; // if the request is missing a required parameter
extern NSString* const MKErrorXMLParsing;   // if the response wasn't valid XML
extern NSString* const MKErrorAPISchema;    // if the response didn't match the API schema
extern NSString* const MKErrorAPIOperation; // if the response was an API error
extern NSString* const MKFacebookAPIVersion;
extern NSString* const MKLoginUrl;
extern NSString* const MKAPIServerURL;
extern NSString* const MKAPISecureServerURL;

@interface MKFacebook : NSObject {
	
	NSString *defaultsName;
	MKLoginWindow *loginWindow;
	NSString *apiKey;
	NSString *secretKey;
	NSString *authToken;
	NSString *sessionKey;
	NSString *sessionSecret;
	NSString *uid;
	BOOL hasAuthToken;
	BOOL hasSessionKey;
	BOOL hasSessionSecret;
	BOOL hasUid;
	BOOL userCancelledLogin;
	
	// API connection state
	BOOL receiveFinished;
	BOOL requestCancelled;
	NSMutableData *receivedData;
	NSString *requestError;			// set to one of the above error strings, or nil
	NSString *errorMethod;		    // if we had MKErrorMissingParam
	NSData *errorData;			    // if we had MKErrorXMLParsing
	NSXMLDocument *errorXML;	    // if we had MKErrorAPISchema
	NSDictionary *errorDictionary; // if we had MKErrorAPIOperation
	BOOL debug;

	NSTimeInterval connectionTimeoutInterval;
	NSError *lastError;
	NSString * facebookAPIVersion;
	id _delegate;
	int facebookDataFormat;
	
}


+(MKFacebook *)facebookWithAPIKey:(NSString *)apiKey withSecret:(NSString *)secretKey withDefaultsName:(NSString *)name delegate:(id)aDelegate;
-(MKFacebook *)initUsingAPIKey:(NSString *)anApi usingSecret:(NSString *)aSecret withDefaultsName:(NSString *)name delegate:(id)aDelegate;

-(NSString *)apiKey;
-(NSString *)sessionKey;
-(BOOL)loadPersistentSession;
-(NSString *)generateSigWithParameters:(NSDictionary *)parameters;
-(NSString *)generateSigWithParameters:(NSDictionary *)parameters version:(int)version;
-(NSString *)generateTimeStamp;

-(NSString *)facebookDataFormat;
-(void)setFacebookDataFormat:(int)aFacebookDataFormat;

-(NSString *)uid;
-(BOOL)userLoggedIn;
-(void)resetFacebookConnection;

//maybe this will be useful in the future
-(void)setFacebookAPIVersion:(NSString *)aFacebookAPIVersion;
-(NSString *)facebookAPIVersion;

//Login Window
-(BOOL)getAuthSession;
-(BOOL)acquireAuthSession;
-(void)showFacebookLoginBrowser;
-(void)showFacebookLoginWindow;
-(NSWindow *)showFacebookLoginWindowForSheet;
-(void)setConnectionTimeoutInterval:(double)aConnectionTimeoutInterval;
-(NSError *)lastError;
-(id)fetchFacebookData:(NSURL *)theURL;

@end

@interface MKFacebook(MKFacebookDelegateMethods)

-(void)userLoginSuccessful;
-(void)userLoginFailed;

@end

@interface NSString (StringExtras)

- (NSString *) encodeURLLegally;

@end