/*
 //  Controller.m
 //  FMenu
 //
 //  Created by Logan Rockmore on 12/16/06.
 Contributors:	Logan Rockmore
				Paul Borokhov
 //  Copyright 2006 by Contributors, 2007 by Paul Borokhov. All rights reserved.
 //
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FMenu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#import "Controller.h"
#import "StringOSVersionComparator.h"

#pragma mark C code

void networkStatusDidChange(SCNetworkReachabilityRef name, SCNetworkConnectionFlags flags, void * infoDictionary) {
	// code from http://git.symonds.id.au/gitweb.cgi?p=marcopolo.git;a=blob;f=src/IPEvidenceSource.m;hb=HEAD used to figure out how to do this correctly. Thanks to David Symonds from cocoa-dev@lists.apple for the help.
	if (name != NULL && infoDictionary != NULL) {
		Controller *info = (Controller*) infoDictionary;
		if (flags != kSCNetworkFlagsReachable) {
			[info setReachable:NO];
		} else {
			[info setReachable:YES];
		}
	}
}

#pragma mark -
@implementation Controller : NSObject

#pragma mark Startup

+ (void)initialize {
    [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:@"6", @"updateTime", @"YES", @"countUnreadMessages", @"NO", @"countUnseenPokes", @"NO", @"countUpcomingEvents", @"NO", @"countFriendRequests", @"NO", @"countPhotoComments", @"NO", @"countShares", @"NO", @"countWallPosts", @"0", @"wallDestination", @"NO", @"dontDisplayImage", @"NO", @"dontDisplay0", @"YES", @"displayQuickLaunch", @"YES", @"showFriendsSub", @"YES", @"showGroupsSub", @"YES", @"showNotesSub", @"YES", @"showPhotosSub", @"YES", @"showMarketSub", @"YES", @"showSharesSub", @"NO", @"logErrors", @"NO", @"logDiagnostics", @"NO", @"logXMLData", @"YES", @"fillFriends", @"YES", @"fillEvents", @"YES", @"fillGroups", @"YES", @"showMessagesSub", @"YES", @"clickContextActive", @"NO", @"logPhotoCommentsData", nil]];
}

- (id) init {
	self = [super init];
	if (self != nil) {
		[self setConstants];
	}
	return self;
}


- (void)awakeFromNib {
	// Initialize status bar item
	statusItem = [[[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength] retain];
	[statusItem setMenu:statusItemMenu];
	[statusItem setImage:[NSImage imageNamed:loggedOutImageName]];
	[statusItem setAlternateImage:[NSImage imageNamed:loggedOutImageNameInv]];
	[statusItem setHighlightMode:YES];

	NSMenuItem *dummyMenu = [[NSMenuItem alloc] initWithTitle:@"" action:NULL keyEquivalent:@""];
	[dummyMenu setSubmenu:statusItemMenu];
	[[NSApp mainMenu] addItem:dummyMenu];
	
	// Start Growl
	growl = [[[GNotifier alloc] init] retain];
	
	// set indents for menu items (does IB really not do this?)
	[totalMessagesMenuItem setIndentationLevel:1];
	[totalPokesMenuItem setIndentationLevel:1];
	[totalEventsMenuItem setIndentationLevel:1];
	[friendsMenuItem setIndentationLevel:1];
	[friendsNotesMenuItem setIndentationLevel:1];
		
	// set up network status change monitoring
	facebookName = SCNetworkReachabilityCreateWithName(kCFAllocatorDefault, [@"www.facebook.com" cStringUsingEncoding:NSISOLatin1StringEncoding]);
	[self setInitialAvailability];
	
	/*SCNetworkReachabilityContext info;
	info.version = 0;
	info.info = self;
	info.retain = NULL;
	info.release = NULL;
	info.copyDescription = NULL;
	
	if (SCNetworkReachabilitySetCallback(facebookName, networkStatusDidChange, &info) &&
		SCNetworkReachabilityScheduleWithRunLoop(facebookName, CFRunLoopGetCurrent(), kCFRunLoopDefaultMode)) {
		// damn, when did we start writing C code?
	}*/

	if (![self checkAvailability]) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"Connection down...");
		}
	} else {
		[self initFacebook];
	}
}

- (void)initFacebook {
	@try {
		facebook = [[MKFacebook facebookWithAPIKey:apikey withSecret:secret withDefaultsName:[[NSBundle mainBundle] bundleIdentifier] delegate:self] retain];
		[NSApp activateIgnoringOtherApps:YES];
		if (![facebook loadPersistentSession]) {
			[facebook showFacebookLoginWindow];
		} else {
			[self userLoginSuccessful];
		}
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"displayQuickLaunch"]) {
			[quickLaunchPanel orderFront:self];
		}
		[quickLaunchPanel setLevel:NSModalPanelWindowLevel];
		[quickLaunchPanel setHidesOnDeactivate:NO];
	} @catch (NSException * e) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"Could not open login page");
		}
	}
}

- (void)userLoginSuccessful {
	if ([facebook userLoggedIn]) {
		// write prefs to disk immediately in case of crashes to save our infinite session (if any)
		[[[NSUserDefaultsController sharedUserDefaultsController] defaults] synchronize];
		[NSThread detachNewThreadSelector:@selector(getProfile:) toTarget:self withObject:nil];
		[self setMenuImage:self];
		[self getCounts];
		[self startTimer:initialDelay];
		[self setLoginMenuItemText];
		// there's got to be a less intensive way of doing this...
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateExplanation) name:NSTextDidChangeNotification object:nil];
		namesArray = [[NSMutableArray arrayWithCapacity:10] retain];
		eventsArray = [[NSMutableArray arrayWithCapacity:10] retain];
		groupsArray = [[NSMutableArray arrayWithCapacity:10] retain];
		quicklaunchArray = [[NSMutableArray arrayWithCapacity:10] retain];
		[self fillsearch:self];
		[nameSearch setDataSource:quicklaunchArray];
	}
}

- (void)userLoginFailed:(id)error {
	// clear all session info and try again
	[facebook resetFacebookConnection];
	if ([self checkAvailability]) {
		[facebook showFacebookLoginWindow];
	}
}

#pragma mark App Methods

- (IBAction)goToFriendProfile:(id)sender {
	id ind;
	if ((ind = [idDic objectForKey:[quicklaunchArray comboBox:nameSearch objectValueForItemAtIndex:[nameSearch indexOfSelectedItem]]]) != nil) {
		[ind launchID];
		[nameSearch deselectItemAtIndex:[nameSearch indexOfSelectedItem]];
	} else {
		[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://www.facebook.com/s.php?q=%@",[[nameSearch stringValue] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]]]];
	}
	[nameSearch setObjectValue:@""];
	[NSApp deactivate];
}

- (void)checkForUpdates:(NSTimer*)theTimer {
	[NSThread detachNewThreadSelector:@selector(checkForUpdatesThread:) toTarget:self withObject:nil];
}

- (void)checkForUpdatesThread:(id)theObject {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	@try {
		NSDictionary *updateDic = [NSDictionary dictionaryWithContentsOfURL:[NSURL URLWithString:@"http://fmenu.sourceforge.net/version.plist"]];
		if ([updateDic count] != 0) {
			NSString *currentVersion = [updateDic valueForKey:@"version"];
			// adapted from code in MEPrefs.m of the Meteo project (http://heat-meteo.sf.net) released under GPL
			NSString *localVersion = [[[NSBundle bundleForClass:[self class]] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
			if (![localVersion isEqual:currentVersion]) {
				NSString *changes = [updateDic valueForKey:@"changes"];
				[growl doNotify:nil type:@"New version available" note:changes];
			}
		} else {
			if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
				NSLog(@"Could not connect to SF.net");
			}
		}
	} @catch (NSException *e) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"Error connecting to SF.net");
		}
	} @finally {
		[pool release];
	}
}

- (void)setInitialAvailability {
	SCNetworkConnectionFlags reachability;
	SCNetworkReachabilityGetFlags(facebookName, &reachability);
	if (reachability != kSCNetworkFlagsReachable) {
		isReachable = NO;
	} else {
		isReachable = YES;
	}
}

- (BOOL)checkAvailability {
	return isReachable;
}

- (void)setReachable:(BOOL)value {
	isReachable = value;
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
		if (value) {
			NSLog(@"Connection back up...");
		} else {
			NSLog(@"Connection down...");
		}
	}
}

#pragma mark "Delegate" methods

- (void)setMenuImage:(id)sender {
	// hide image if so desired
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"dontDisplayImage"]) {
		[statusItem setImage:nil];
	} else {
		if ([facebook userLoggedIn]) {
			if (sleeping) {
				[statusItem setImage:[NSImage imageNamed:sleepingImageName]];
			} else {
				[statusItem setImage:[NSImage imageNamed:loggedInImageName]];
				[statusItem setAlternateImage:[NSImage imageNamed:loggedInImageNameInv]];
			}
		} else {
			[statusItem setImage:[NSImage imageNamed:loggedOutImageName]];
			[statusItem setAlternateImage:[NSImage imageNamed:loggedOutImageNameInv]];
		}
	}
}

- (void)setLoginMenuItemText {
	if ([facebook userLoggedIn]) {
		[resetMenuItem setTitle:NSLocalizedStringFromTable(@"Log out...", nil, nil)];
	} else {
		[resetMenuItem setTitle:NSLocalizedStringFromTable(@"Log in...", nil, nil)];
	}
}

- (void)menuNeedsUpdate:(NSMenu *)menu {
	[self createSubmenus];
}

- (void)windowDidClose:(NSNotification *)aNotification {
	if ([aNotification object] == preferencesWindow && [preferencesWindow isVisible]) { // why do we need to check for visibility??
		[self restartTimers];
		[[[NSUserDefaultsController sharedUserDefaultsController] defaults] synchronize];
        [self fillsearch:self];
	}
}

- (void)windowDidBecomeKey:(NSNotification *)aNotification {
	if ([aNotification object] == quickLaunchPanel) {
		[nameSearch selectText:self];
	}
}

- (void)applicationWillTerminate:(NSNotification *)aNotification {
	CFRelease(facebookName);
	[timer invalidate];
	[[NSRunLoop currentRunLoop] cancelPerformSelectorsWithTarget:self];
	[oldData release];
	[idDic release];
}

- (SUAppcastItem *)bestValidUpdateInAppcast:(SUAppcast *)appcast forUpdater:(SUUpdater *)bundle {
    // need to grab the SUAppcastItem which has the closest minimum version to ours
    NSArray *items = [appcast items];
    NSString *currentOS = @"10.3";
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_OS_X_VERSION_10_6
    currentOS = @"10.6";
#elif __MAC_OS_X_VERSION_MAX_ALLOWED < __MAC_OS_X_VERSION_10_6
    currentOS = @"10.5";
#elif __MAC_OS_X_VERSION_MAX_ALLOWED < __MAC_OS_X_VERSION_10_5
    currentOS = @"10.4";
#endif
    if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) NSLog(@"got current OS as %@", currentOS);
    SUAppcastItem *chosen = nil;
    for (int i = [items count]-1; i>=0; i--) {
        SUAppcastItem *item = [items objectAtIndex:i];
        NSString *osversion = [item minimumSystemVersion];
        if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) NSLog(@"Considering update with minversion %@, verion %@, disp version %@", osversion, [item versionString], [item displayVersionString]);
        if (!chosen && ([currentOS sameVersion:osversion] || [currentOS higherVersion:osversion])) {
            if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) NSLog(@"item OS <= current OS, remembering item");
            chosen = item;
        }
        else if (chosen && [osversion higherVersion:[chosen minimumSystemVersion]] && ([osversion sameVersion:currentOS] || [osversion lowerVersion:currentOS])) {
            if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) NSLog(@"item OS is higher than previously chosen and meets minimum reqs");
            chosen = item;
        } else if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) NSLog(@"item doesn't satisfy any reqs, continuing");
    }
    return chosen;
}

@end