//
//  PreferenceFunctions.m
//  FMenu
//
//  Created by Paul Borokhov on 1/5/07.
//  Copyright 2007 Paul Borokhov. All rights reserved.
/*
 **
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FMenu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#import "PreferenceFunctions.h"
#define GROWLPATH @"PreferencePanes/Growl.prefPane"
#define CONSOLE_ID @"com.apple.Console"

@implementation Controller (PreferenceFunctions)

#pragma mark -
#pragma mark Preferences

- (void)windowDidBecomeMain:(NSNotification *)aNotification {
	if ([aNotification object] == preferencesWindow) {
		// damn you IB and NSTabView ... I want a specific keyboard loop, not the automatic one.
		// [[prefsTabView tabViewItemAtIndex:0] setInitialFirstResponder:initialFocusItem];
		[self updateExplanation];
		if ([self detectGrowl:openGrowl]) {
			[openGrowl setEnabled:YES];
			[openGrowl setTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"Growl preferences...", nil, nil)]];
		} else {
			[openGrowl setEnabled:NO];
			[openGrowl setTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"Growl not found!", nil, nil)]];
		}
	}
}

- (BOOL)detectGrowl:(id)anItem {
	if ([anItem action] == @selector(openGrowl:)) {
		unsigned char success = 0;
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSAllDomainsMask, YES);
		for (int i = 0; i<[paths count]; i++) { // 4 domains possible...there needs to be a better way of doing this
			if ([[NSFileManager defaultManager] directoryContentsAtPath:[[paths objectAtIndex:i] stringByAppendingPathComponent:GROWLPATH]] != nil) {
				success++;
				break;
			}
		}
		if (success == 1) { // this is a lame way of doing this, but better than nothing for now
			return YES;
		} else {
			return NO;
		}
	} else {
		return YES;
	}
}

- (void)updateExplanation {
	[timerExplanation setTitleWithMnemonic:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"FMenu performs %i kinds of updates at the interval you specify above. A complete update cycle therefore takes %i seconds.", nil, nil), noOfChecks, [[[[refreshInterval cell] formatter] numberFromString:[refreshInterval stringValue]] intValue] * noOfChecks]];
}

- (void)openGrowl:(id)sender {
	/* GAHHHH WHY WON'T THIS WORK? sure it's not an App, but it would make things a hell of a lot easier...
	if (![[NSWorkspace sharedWorkspace] launchAppWithBundleIdentifier:growlID options:NSWorkspaceLaunchWithoutAddingToRecents additionalEventParamDescriptor:nil launchIdentifier:nil]) {
		NSLog(@"Couldn't find Growl...");
	}*/
	unsigned char success = 0;
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSAllDomainsMask, YES);
    for (int i = 0; i<[paths count]; i++) { // 4 domains possible...there needs to be a better way of doing this
		if ([[NSWorkspace sharedWorkspace] openFile:[[paths objectAtIndex:i] stringByAppendingPathComponent:GROWLPATH]]) {
			success++;
			break;
		}
	}
	if (success == 1) {
		[[sender window] performClose:sender];
	}
}

- (void)openConsole:(id)sender {
	if (![[NSWorkspace sharedWorkspace] launchAppWithBundleIdentifier:CONSOLE_ID options:NSWorkspaceLaunchWithoutAddingToRecents additionalEventParamDescriptor:nil launchIdentifier:nil]) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"Couldn't find Console.app...");
		}
	}
}

@end
